#include "mgn.h"
#include "dtr.h"
#include "dtr_surf.h"

main(int argc, char *argv[])
{
  double **coords;
  int np;
  int tp;
  int nedgs;
  int **edgs;
  int nloops;
  int **loops;
  int *lloops;
  int nsurfs;
  SURF *surfs;
  int **tr;
  int ntr;

  FILE *fp;
  double mprval;
  int i, j, e, s, c, l, n;
  char line[E_LINE_MAX];
  char *pline;
  char word[E_LINE_MAX];
  char filename[E_LINE_MAX];
  char temp[E_LINE_MAX];
  int ln;
  int ne, nl, lp;

  MESH *m;

  PrintProgress = TRUE;
  ShowGraphics = FALSE;

  if (argc != 3)
  {
    printf("usage: surftest file 0.1\n\n");
    exit(1);
  }

  /* read and parse the rgm file */
  sprintf(filename, "%s.rgm", argv[1]);
  if ((fp = fopen(filename, "r")) == NULL)
    e_GeneralError("can't open rgm file");

  sscanf(argv[2], "%lf", &mprval);

  ln = 0;
  GetKeyword(line, word, &ln, filename, fp);
  if (strcmp(word, "vertices") != 0)
    e_LineWarning(ln, "rgm file not as expected (1)");
  sscanf(line, "%*s %d", &np);
  coords = e_alloc_cont_d_matr(np, 3);
  tp = np;

  for (n = 0; n < np; n++)
  {
    e_GetNextLine(line, &ln, filename, fp);
    sscanf(line, "%*s %lf %lf %lf",
           &(coords[n][0]), &(coords[n][1]), &(coords[n][2]));
  }

  GetKeyword(line, word, &ln, filename, fp);
  if (strcmp(word, "edges") != 0)
    e_LineWarning(ln, "rgm file not as expected (2)");
  sscanf(line, "%*s %d", &nedgs);
  edgs = e_alloc_cont_i_matr(nedgs, 2);

  for (e = 0; e < nedgs; e++)
  {
    e_GetNextLine(line, &ln, filename, fp);
    sscanf(line, "%*s v%d v%d", &(edgs[e][0]), &(edgs[e][1]));
    edgs[e][0]--;
    edgs[e][1]--;
  }

  GetKeyword(line, word, &ln, filename, fp);
  if (strcmp(word, "loops") != 0)
    e_LineWarning(ln, "rgm file not as expected (3)");
  sscanf(line, "%*s %d", &nloops);
  loops = e_alloc_cont_i_matr(nloops, 50);
  lloops = e_alloc_i_vect(nloops);

  for (l = 0; l < nloops; l++)
  {
    e_GetNextLine(line, &ln, filename, fp);
    pline = line;
    pline = GetStrArg(pline, "%s", &temp);
    pline = GetStrArg(pline, "%d", &ne);
    lloops[l] = ne;
    for (e = 0; e < ne; e++)
    {
      pline = GetStrArg(pline, "e%d", &(loops[l][e]));
      loops[l][e]--;
    }
  }

  GetKeyword(line, word, &ln, filename, fp);
  if (strcmp(word, "surfaces") != 0)
    e_LineWarning(ln, "rgm file not as expected (4)");
  sscanf(line, "%*s %d", &nsurfs);

  surfs = (SURF *) e_calloc(nsurfs, sizeof(SURF));
  for (s = 0; s < nsurfs; s++)
  {
    e_GetNextLine(line, &ln, filename, fp);
    pline = line;
    pline = GetStrArg(pline, "%s", &temp);
    pline = GetStrArg(pline, "%d", &nl);
    for (l = 0; l < nl; l++)
    {
      pline = GetStrArg(pline, "l%d", &lp);
      lp--;
      if (l == 0)
      {
        surfs[s].nil = nl - 1;
        surfs[s].ol.ne = lloops[lp];
        surfs[s].ol.e = e_alloc_i_vect(lloops[lp]);
        for (e = 0; e < lloops[lp]; e++)
          surfs[s].ol.e[e] = loops[lp][e];
      }
      else
      {
        if (l == 1)
          surfs[s].il = (ELOOP *) e_calloc(nl - 1, sizeof(ELOOP));
        surfs[s].il[l - 1].ne = lloops[lp];
        surfs[s].il[l - 1].e = e_alloc_i_vect(lloops[lp]);
        for (e = 0; e < lloops[lp]; e++)
          surfs[s].il[l - 1].e[e] = loops[lp][e];
      }
    }
  }
  fclose(fp);

  /* ok here we go ! */
  refine_surface(&coords, &np, &tp, mprval, nedgs, edgs, nsurfs,
  							 surfs, &tr, &ntr);

  /* and output */
  m = (MESH *) e_calloc(1, sizeof(MESH));
  e_NullifyMesh(m);
  create_outputmesh(coords, np, tr, ntr, m);

  e_PutMeshData(m, argv[1]);

  /* freeup */
  e_FreeMeshData(m);
  e_free_cont_d_matr(tp, coords);
  e_free_cont_i_matr(nedgs, edgs);
  e_free_cont_i_matr(nloops, loops);
  e_free(lloops);
  for (s = 0; s < nsurfs; s++)
  {
    e_free(surfs[s].ol.e);
    for (i = 0; i < surfs[s].nil; i++)
      e_free(surfs[s].il[i].e);
    e_free(surfs[s].il);
  }
  e_free(surfs);
  e_free_cont_i_matr(ntr, tr);
}
